package com.ichaoj.ams.controller;

import cn.hutool.core.bean.BeanUtil;
import com.ichaoj.ams.entity.AmsAuto;
import com.ichaoj.ams.request.auto.CreateAuto;
import com.ichaoj.ams.request.auto.PageAutoRequest;
import com.ichaoj.ams.request.auto.UpdateAuto;
import com.ichaoj.ams.response.auto.AutoResponse;
import com.ichaoj.ams.service.IAmsAutoService;
import com.ichaoj.common.annotation.AuthResource;
import com.ichaoj.common.model.PublicPage;
import com.ichaoj.common.model.PublicResult;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * @author : cjwen
 * @date : 2023/05/10 10:21
 */
@RestController
@RequestMapping("/auto")
@Tag(name = "自动化交互")
public class AutoController {


    @Resource
    private IAmsAutoService autoService;

    @PostMapping
    @Operation(summary = "创建自动化")
    @AuthResource(true)
    public PublicResult<Object> createAuto(@RequestBody CreateAuto createAuto) {
        autoService.createAuto(createAuto);
        return PublicResult.success();
    }

    @PutMapping
    @Operation(summary = "修改自动化")
    @AuthResource(true)
    public PublicResult<Object> updateAuto(@RequestBody @Validated UpdateAuto updateAuto) {
        autoService.updateAuto(updateAuto);
        return PublicResult.success();
    }

    @DeleteMapping("/{autoId}")
    @Operation(summary = "删除自动化")
    @AuthResource(true)
    public PublicResult<Object> deleteAuto(@PathVariable String autoId) {
        autoService.removeById(autoId);
        return PublicResult.success();
    }

    @PostMapping("/page")
    @Operation(summary = "分页查询自动化")
    @AuthResource
    public PublicResult<PublicPage<AutoResponse>> pageAuto(@RequestBody PageAutoRequest autoRequest) {
        PublicPage<AutoResponse> result = autoService.pageAuto(autoRequest);
        return PublicResult.success(result);
    }
    
}
