package com.ichaoj.ams.controller;

import cn.hutool.core.bean.BeanUtil;
import com.ichaoj.ams.entity.AmsStrategy;
import com.ichaoj.ams.request.strategy.CreateStrategy;
import com.ichaoj.ams.request.strategy.PageStrategyRequest;
import com.ichaoj.ams.request.strategy.UpdateStrategy;
import com.ichaoj.ams.response.strategy.StrategyResponse;
import com.ichaoj.ams.service.IAmsStrategyService;
import com.ichaoj.common.annotation.AuthResource;
import com.ichaoj.common.model.PublicPage;
import com.ichaoj.common.model.PublicResult;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.time.LocalDateTime;

/**
 * <p>
 * 策略管理 前端控制器
 * </p>
 *
 * @author cjwen
 * @since 2023-04-25
 */
@RestController
@RequestMapping("/strategy")
@Tag(name = "策略模块")
public class StrategyController {

    @Resource
    private IAmsStrategyService strategyService;

    @PostMapping
    @Operation(summary = "创建策略")
    @AuthResource(true)
    public PublicResult<Object> createStrategy(@RequestBody CreateStrategy createStrategy) {
        AmsStrategy strategy = BeanUtil.copyProperties(createStrategy, AmsStrategy.class);
        strategy.setCreateTime(LocalDateTime.now());
        strategyService.save(strategy);
        return PublicResult.success();
    }

    @PutMapping
    @Operation(summary = "修改策略")
    @AuthResource(true)
    public PublicResult<Object> updateStrategy(@RequestBody @Validated UpdateStrategy updateStrategy) {
        AmsStrategy strategy = BeanUtil.copyProperties(updateStrategy, AmsStrategy.class);
        strategy.setUpdateTime(LocalDateTime.now());
        strategyService.updateById(strategy);
        return PublicResult.success();
    }

    @DeleteMapping("/{strategyId}")
    @Operation(summary = "删除策略")
    @AuthResource(true)
    public PublicResult<Object> deleteStrategy(@PathVariable String strategyId) {
        strategyService.removeById(strategyId);
        return PublicResult.success();
    }

    @PostMapping("/page")
    @Operation(summary = "分页查询策略")
    @AuthResource
    public PublicResult<PublicPage<StrategyResponse>> pageStrategy(@RequestBody PageStrategyRequest strategyRequest) {
        PublicPage<StrategyResponse> result = strategyService.pageStrategy(strategyRequest);
        return PublicResult.success(result);
    }

}
