package com.ichaoj.ams.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ichaoj.ams.constant.AmsConstant;
import com.ichaoj.ams.entity.AmsAirdropProject;
import com.ichaoj.ams.entity.AmsAirdropTask;
import com.ichaoj.ams.mapper.AmsAirdropProjectMapper;
import com.ichaoj.ams.request.airdrop.CreateAirdropProject;
import com.ichaoj.ams.request.airdrop.PageProjectRequest;
import com.ichaoj.ams.request.airdrop.UpdateAirdropProject;
import com.ichaoj.ams.response.airdrop.AirdropProjectResponse;
import com.ichaoj.ams.service.IAmsAirdropProjectService;
import com.ichaoj.ams.service.IAmsAirdropTaskService;
import com.ichaoj.common.model.PublicPage;
import com.ichaoj.mybatis.service.SuperWhaleServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 空投项目 服务实现类
 * </p>
 *
 * @author cjwen
 * @since 2023-05-10
 */
@Service
public class AmsAirdropProjectServiceImpl extends SuperWhaleServiceImpl<AmsAirdropProjectMapper, AmsAirdropProject> implements IAmsAirdropProjectService {

    @Resource
    private IAmsAirdropTaskService taskService;

    @Override
    public PublicPage<AirdropProjectResponse> pageAirdropProject(PageProjectRequest projectRequest) {
        LambdaQueryWrapper<AmsAirdropProject> wrapper = Wrappers.lambdaQuery();
        wrapper
                .eq(StrUtil.isNotBlank(projectRequest.getQueryValue()), AmsAirdropProject::getProjectWebsite, projectRequest.getQueryValue())
                .eq(StrUtil.isNotBlank(projectRequest.getQueryValue()), AmsAirdropProject::getProjectUrl, projectRequest.getQueryValue())
                .or().like(StrUtil.isNotBlank(projectRequest.getQueryValue()), AmsAirdropProject::getProjectName, projectRequest.getQueryValue())
                .orderByDesc(AmsAirdropProject::getCreateTime);
        Page<AmsAirdropProject> result = this.page(this.buildPageObj(projectRequest), wrapper);
        return this.convertPublicPage(result, s -> BeanUtil.copyProperties(s, AirdropProjectResponse.class));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createAirdropProject(CreateAirdropProject createAirdropProject) {
        AmsAirdropProject project = BeanUtil.copyProperties(createAirdropProject, AmsAirdropProject.class);
        project.setCreateTime(LocalDateTime.now());
        project.setStatus(AmsConstant.AirdropStatus.NOT_STARTED);
        this.save(project);
        if (CollectionUtil.isNotEmpty(createAirdropProject.getTasks())) {
            List<AmsAirdropTask> tasks = BeanUtil.copyToList(createAirdropProject.getTasks(), AmsAirdropTask.class);
            tasks.forEach(t -> {
                t.setAirdropProjectId(project.getAmsProjectId());
                t.setCreateTime(LocalDateTime.now());
                t.setTaskStatus(0);
            });
            taskService.saveBatch(tasks);
        }

    }

    @Override
    public void updateAirdropProject(UpdateAirdropProject updateAirdropProject) {
        AmsAirdropProject project = BeanUtil.copyProperties(updateAirdropProject, AmsAirdropProject.class);
        project.setUpdateTime(LocalDateTime.now());
        this.updateById(project);
        if (CollectionUtil.isNotEmpty(updateAirdropProject.getTasks())) {
            List<AmsAirdropTask> tasks = BeanUtil.copyToList(updateAirdropProject.getTasks(), AmsAirdropTask.class);
            tasks.forEach(t -> {
                t.setAirdropProjectId(project.getAmsProjectId());
                t.setUpdateTime(LocalDateTime.now());
            });
            taskService.updateBatchById(tasks);
        }
    }

}
